class Event(object):
    """this is a superclass for any events that might be generated by an
    object and sent to the EventManager"""
    def __init__(self):
        self.name = "Generic Event"

class ObjectRefreshEvent(Event):
    def __init__(self, obj):
        self.obj = obj

class ObjectDestroyEvent(Event):
    def __init__(self, obj, killstr=None, remains=False, chrome=True):
        # set chrome to False to disable corpses and whatnot
        self.obj = obj
        self.killstr = killstr
        self.remains = remains
        self.chrome = chrome

class ObjectDestroyPartEvent(Event):
    def __init__(self, obj):
        self.obj = obj

class ObjectHitEvent(Event):
    def __init__(self, obj, killstr, dies=False):
        self.obj = obj
        self.killstr = killstr
        self.dies = dies

class ObjectIncomingEvent(Event):
    def __init__(self, obj):
        self.obj = obj

class ExplosionSpriteEvent(Event):
    def __init__(self, *args):
        self.args = args

class AddImageEvent(Event):
    def __init__(self, layer, pos, strip, rot=0, attr=None, rel=None, start=None, texture=None, repeat=True):
        self.layer = layer
        self.pos = pos
        self.strip = strip
        self.rot = rot
        self.attr = attr
        self.rel = rel
        self.texture = texture
        self.start = start
        self.repeat = repeat

class AddChainEvent(Event):
    def __init__(self, obj1, obj2, pos1, pos2, txt, rot, start=0):
        self.obj1 = obj1
        self.obj2 = obj2
        self.pos1 = pos1
        self.pos2 = pos2
        self.txt = txt
        self.rot = rot
        self.start = start

class GeomBGEvent(Event):
    def __init__(self, geom, image=None, texture=None):
        self.geom = geom
        self.image = image
        self.texture = texture

class FXEvent(Event):
    def __init__(self, part, type, kwargs):
        self.part = part
        self.type = type
        self.kwargs = kwargs

class ObjectShootingEvent(Event):
    def __init__(self, obj):
        self.obj = obj

class NewSoundEvent(Event):
    def __init__(self, filename, loop=False, music=False):
        self.file = filename
        self.loop = loop
        self.music = music

class StopSoundEvent(Event):
    def __init__(self, filename):
        self.file = filename

class ScreenshotEvent(Event):
    pass

class PauseEvent(Event):
    pass

