import math
import random #For drawing random backgrounds
import pygame
import operator
import Event
import Image
import Vector
import Numeric
import Game
import Level
import Goals
from data import Constants
from data.Constants import *

movie = False #Can get set from outside in order to trigger writing movie.

def fill_texture(size, rot, txt):
    '''Pattern a surface with repeated versions of image.'''
    w, h = size
    tw, th = txt.get_size()
    if rot % 180:
        h, w = w, h
    image = pygame.Surface((int(w), int(h)))
    if txt.get_colorkey():
        image.fill(txt.get_colorkey())
        image.set_colorkey(txt.get_colorkey())
    else:
        pink = (0, 255, 255)
        image.fill(pink)
        image.set_colorkey(pink)
    for i in range(0, int(w), tw):
        for j in range(0, int(h), th):
            image.blit(txt, (i, j))
    image = pygame.transform.rotate(image, -rot)
    return image

# alexandria sprite classes -- necessary to keep track of Strip drawing
class StripSprite(pygame.sprite.Sprite):
    repeat = True
    def __init__(self, start=None):
        super(StripSprite, self).__init__()
        if start == None: start = 0
        self.n = start
        self.offset = (0, 0)
        self.strip = None

    def get_image(self):
        return self.strip[self.n]

    def get_pos(self):
        return (0, 0)

    def destroy(self):
        '''Action to take when destroyed (if not cyclic).'''
        pass

    def get_rect(self):
        r = self.get_image().get_rect()
        r.center = self.get_pos()
        r.move_ip(self.offset)
        return r

    def update(self, offset=None):
        if offset:
            self.offset = offset
        else:
            self.n += 1
            if self.n >= len(self.strip):
                if self.repeat == False:
                    self.kill()
                    self.destroy()
                if self.repeat != True:
                    self.n = self.repeat

    image = property(lambda self: self.get_image())
    rect = property(lambda self: self.get_rect())

class FadeSprite(pygame.sprite.Sprite):
    def __init__(self, size, color, ticks, reverse=False):
        super(FadeSprite, self).__init__()
        self.surface = pygame.surface.Surface(size)
        self.surface.fill(color)
        self.reverse = reverse
        alpha = 0
        if reverse: alpha = 255
        self.surface.set_alpha(alpha)
        self.image = self.surface
        self.rect = pygame.rect.Rect(0, 0, 0, 0)
        self.i = 0
        self.ticks = ticks
    def update(self, *args):
        #if self.i == self.ticks: self.kill()
        if len(args) == 0:
            alpha = 255*self.i/self.ticks
            if self.reverse: alpha = 255 - alpha
            self.image.set_alpha(alpha)
            if self.i < self.ticks:
                self.i += 1
    
class ZoomSprite(pygame.sprite.Sprite):
    def __init__(self, image=None, rotrate=None, startpt=None, velocity=None):
        super(ZoomSprite, self).__init__()
        self.orig_image = image
        self.rotrate = rotrate
        self.velocity = velocity
        self.rot = 0
        self.pos = startpt[:-1]
        self.z = startpt[-1]
        self.n = 0
        self.render()

    def update(self, *args):
        if len(args) == 0:
            self.render()
            x, y, z = self.velocity
            self.pos = self.pos[0]+x, self.pos[1]+y
            self.z += z
            self.rot += self.rotrate
            self.n += 1

    def render(self):
        self.image = pygame.transform.rotozoom(self.orig_image[self.n], self.rot,
                                               1.0/self.z)
        #alphas = pygame.surfarray.pixels_alpha(self.image)
        #alphas[:, :, :] = Numeric.bitwise_and(alphas, pygame.surfarray.array_colorkey(self.image))
        self.image.convert()
        #print self.image.get_colorkey(), self.orig_image[self.n].get_colorkey()
        self.image.set_colorkey(self.orig_image[self.n].get_colorkey())
        #print self.image.get_colorkey()
        #pygame.image.save(self.image, 'junket-%03d.tga'%self.n)
        self.rect = self.image.get_rect()
        self.rect.center = self.pos[0]/self.z, self.pos[1]/self.z


class BGSprite(StripSprite):
    repeat = True
    def __init__(self, size):
        super(BGSprite, self).__init__()
        self.size = size
        self.state = True
        w, h = self.size
        self.stars = []
        for i in range(100):
            x, y = random.randint(0, w), random.randint(0, h)
            r = random.randint(0, 3)
            c = random.randint(1, 255)
            self.stars.append((x, y, r, c))
        self.draw()
        self.precompute_invert()
        
    def get_rect(self):
        return pygame.rect.Rect(0, 0, *self.size)

    def draw(self):
        image = pygame.surface.Surface(self.size)
        bgcolor = (0,0,0)
        image.fill(bgcolor)
        for x, y, r, c in self.stars:
            pygame.draw.circle(image, (c, c, c), (x, y), r)
        self.strip = Image.Strip([image])

    def precompute_invert(self):
        darkimgs = []
        lightimgs = []
        n = 10
        image = self.strip[-1]
        image = image.convert(32)
        array = pygame.surfarray.pixels3d(image)
##        for i in range(1, n+1):
##            image = pygame.surfarray.make_surface((float(i)/n * (255-2*array.astype('l')) + array).astype(array.typecode()))
##            imgs.append(image)
        darkimgs.append(image)
        last = 255 - array
        lightimgs.append(pygame.surfarray.make_surface(last))
        neutral = pygame.surface.Surface(self.size)
        neutralcolor = (127, 127, 127)
        neutral.fill(neutralcolor)
        neutral.set_alpha(127)
        for i in range(5):
            d = darkimgs[-1].copy()
            d.blit(neutral, (0, 0))
            darkimgs.append(d)
            l = lightimgs[-1].copy()
            l.blit(neutral, (0, 0))
            lightimgs.append(l)
                   
        lightimgs.reverse()
        imgs = darkimgs+lightimgs
        imgs = [i.convert() for i in imgs]
        copy = imgs[:]
        copy.reverse()
        self.invertstrips = [Image.Strip(imgs), Image.Strip(copy)]
        
    
class FlashSprite(pygame.sprite.Sprite):
    def __init__(self, obj, geom, sprite):
        super(FlashSprite, self).__init__()
        self.obj = obj
        self.target = sprite

    def get_image(self):
        img = self.target.image
        array = pygame.surfarray.array3d(img)
        array[:, :, 0] = array[:, :, 1] = array[:, :, 2] = pygame.surfarray.array_colorkey(img)
        surface = pygame.surfarray.make_surface(array)
        surface.set_colorkey((0, 0, 0))
        return surface

    image = property(get_image)

    def get_rect(self):
        return self.target.rect
    rect = property(get_rect)

    def update(self, offset=None):
        if offset == None:
            self.kill()
            pass

class ImageSprite(StripSprite):
    def __init__(self, pos, strip, rot=None, attr=None, rel=None, repeat=True,
                 **kwargs):
        super(ImageSprite, self).__init__(**kwargs)
        if attr == None:
            attr = 'topleft'
        if rot:
            def rotate(i):
                return pygame.transform.rotate(i, -rot)
            self.strip = Image.Strip(map(rotate, strip))
        else:
            self.strip = strip
        r = strip[0].get_rect()
        setattr(r, attr, pos)
        self._pos = r.center
        self._rel = rel
        self.repeat = repeat
    def get_pos(self):
        if not self._rel:
            return self._pos
        else:
            relpos = self._rel.getPosition()
            #print relpos, self._pos
            return relpos[0]+self._pos[0], relpos[1]+self._pos[1]

class RotSprite(StripSprite):
    debug = False
    def __init__(self):
        super(RotSprite, self).__init__()

    def get_image(self):
        if self.debug:
            print self.n,
            print self.strip[self.n]
            print self.strip.files[self.n]
        return pygame.transform.rotate(self.strip[self.n], self.get_angle())

class InSprite(RotSprite):
    '''Sprite to represent something coming onto the playing field.'''
    repeat = False
    def __init__(self, obj, angle, game):
        super(InSprite, self).__init__()
        self.strip = obj.instrip
        self.center = obj.pos
        self.game = game

    def destroy(self):
        self.object.place(self.game)

    def get_pos(self):
        return self.center
    def get_angle(self):
        return self.angle

class Explosion(RotSprite):
    repeat = False
    def __init__(self, center, angle, expfile = 'explosion'):
        super(Explosion, self).__init__()
        self.strip = Image.make_strip(expfile, True)
        self.center = center
        self.angle = angle

    def get_pos(self):
        return self.center

    def get_angle(self):
        return self.angle
    
class ThrustSprite(RotSprite):
    repeat = False
    def __init__(self, angle, pos):
        super(ThrustSprite, self).__init__()
        self.strip = Image.make_strip('thrust', True)
        self.angle = angle
        self.pos = pos

    def get_angle(self):
        return self.angle

    def get_pos(self):
        return self.pos
        
class GeomSprite(RotSprite):
    gimage = None
    texture = None
    def __init__(self, geom, image=None, texture=None, repeat=True):
        self.gimage = image
        self.texture = texture
        self.repeat = repeat
        super(GeomSprite, self).__init__()
        self.body = geom.getBody()
        self.object = geom.object
        # g is the geom to render
        if geom.shape == 'transform':
            g = geom.getGeom()
            self.trans = g.getPosition()[:2]
        else:
            self.trans = None
            g = geom
        self.g = g
        self.getparams(self.g)
        self.render(self.g)
        self.orig_strip = self.strip

    def render(self, g):
        if self.gimage:
            self.strip = self.gimage
        elif self.texture:
            s = [fill_texture(g.size, g.rot, i) for i in self.texture]
            self.strip = Image.Strip(s)
        else:
            if g.shape == 'circle':
                r = g.radius
                image = pygame.Surface((2*r, 2*r))
                image.fill((10,10,10))
                pygame.draw.circle(image, (200, 100, 100), (r, r), r)
                
            if g.shape == 'box':
                image = pygame.Surface(g.size)
                image.fill((255, 255, 255))
            image.set_colorkey((10, 10, 10))
            self.strip = Image.Strip([image])

    def get_pos(self):
        if self.trans:
            return map(lambda x, y:x+y, self.trans,
                       self.body.getPosition()[:2])
        return self.body.getPosition()[:2]

    def get_angle(self):
        try:
            return self.object.angle
        except:
            return math.asin(self.body.getRotation()[1])

    def switch(self, newstrip):
        self.strip = newstrip
        self.n = 0

    def update(self, offset=None):
        super(GeomSprite, self).update(offset=offset)
        if self.n == len(self.strip) and self.strip != self.orig_strip:
            self.strip = self.orig_strip
            self.n = 0

    def refresh(self):
        self.gimage = self.texture = None
        self.getparams(self.g)
        self.render(self.g)
        self.orig_strip = self.strip

    def getparams(self, g):
        if self.gimage or self.texture: return
        
        # get image from transform geom -- is this where it will be placed?
        if hasattr(g, 'image') and g.image:
            self.gimage=g.image
            
        elif hasattr(g, 'texture') and g.texture:
            self.texture = g.texture
        #print g, g.image

def draw_rope(p1, p2, texture, offset, rot=0):
    d = math.sqrt((p1[0]-p2[0])*(p1[0]-p2[0]) + (p1[1]-p2[1])*(p1[1]-p2[1]))
    v = p2[0]-p1[0], p2[1]-p1[1]
    if rot % 180:
        h = texture.get_width()
    else:
        h = texture.get_height()
    image = fill_texture((d, h), rot, texture)
    image = pygame.transform.rotate(image, -math.degrees(math.atan2(v[1], v[0])))
    imagerect = image.get_rect()
    center = (p1[0]+p2[0])/2, (p1[1]+p2[1])/2
    imagerect.center = map(operator.add, center, offset)
    return (image, imagerect)

class ChainSprite(pygame.sprite.Sprite):
    def __init__(self, body1, body2, pos1, pos2, txt, rot, start = 0):
        super(ChainSprite, self).__init__()
        self.body1 = body1
        self.body2 = body2
        self.pos1 = pos1
        self.pos2 = pos2
        self.txt = txt
        self.rot = rot
        self.n = start

    def update(self, offset=None):
        if offset:
            self.regen(offset)
        else:
            self.n += 1

    def regen(self, offset):
        p1 = map(operator.add, self.body1.getPosition()[:2], self.pos1)
        p2 = map(operator.add, self.body2.getPosition()[:2], self.pos2)
        self.image, self.rect = draw_rope(p1, p2, self.txt[self.n], offset, self.rot)

class View(object):
    fx_funcs = {}
    def __init__(self, game):
        pygame.display.init()
        pygame.font.init()
        self.screen = pygame.display.set_mode([800, 600])
        #player = self.player = game.player
        self.game = game
        self.background = BGSprite(self.screen.get_size())
        #pygame.surface.Surface((self.screen.get_width(),
        #                                          self.screen.get_height()))
        #load_background(self.background)

        self.i = 0
        self.font = pygame.font.Font("freesansbold.ttf", 16)
        self.healthfont = pygame.font.Font("freesansbold.ttf", 33)
        self.ropestrip = Image.make_strip('ropebeam', True)
        self.ropehookstrip = Image.make_strip('beamhook', True)
        self.thrustsprites = pygame.sprite.Group()
        self.geom_sprites = {}
        self.spritesgroup = pygame.sprite.Group()
        self.cornersprites = pygame.sprite.Group()
        self.explodesprites = pygame.sprite.Group()
        self.flashsprites = pygame.sprite.Group()
        self.in_sprites = {}
        self.layers = {-1: pygame.sprite.OrderedUpdates(), 0: pygame.sprite.OrderedUpdates(), 1: pygame.sprite.OrderedUpdates(),
                              2: pygame.sprite.OrderedUpdates()}

    def kill_all(self):
        for key, value in self.layers.iteritems():
            for sprite in value:
                sprite.kill()

    def thread(self):
        '''Infinite loop that just calls draw.'''
        while True:
            self.draw()
            yield None
            
    def draw_bg(self):
        self.screen.blit(self.background.image, self.background.rect)
        self.background.update()

    def draw(self):
        self.draw_bg()
        geom = self.game.geometry
        offset = Vector.Vector([400, 300])-geom.get_pos(self.game.player)
        geoms = geom.get_shapes()

        dyngroups = {LAYERBG: [], LAYERGEOMS: [self.thrustsprites, self.spritesgroup], LAYERABOVEGEOMS: [], LAYEREXPLOSIONS: [self.explodesprites, self.flashsprites]}
        for i in range(LAYERBG, LAYEREXPLOSIONS+1):
            for group in [self.layers[i]]:#+dyngroups[i]:
                group.update(offset)
                group.draw(self.screen)
                group.update()

        # Drawing ropes can't be done in sprites because there is a variable
        # number of segments per rope. As such, it is easier to do right here.
        for r in self.game.ropes.values():
            segs = r.segments()
            p0 = segs[0][0]
            hooki = self.ropehookstrip[self.i]
            hookrect = hooki.get_rect()
            hookrect.center = p0
            hookrect.move_ip(offset)
            self.screen.blit(hooki, hookrect)
            for s in segs:
                p1, p2 = s
                image, imagerect = draw_rope(p1, p2, self.ropestrip[self.i], offset)
                
                self.screen.blit(image, imagerect)

        self.draw_health()
        self.draw_goals(offset)
        self.draw_fps()

        if self.game.thrustpos:
            if self.i % 12 == 0:
                s = ThrustSprite(self.game.player.angle, self.game.thrustpos)
                self.thrustsprites.add(s)
                self.layers[Constants.LAYERGEOMS].add(s)
            self.game.thrustpos = None
            
        #s = pygame.transform.rotate(self.playersprite, self.game.player.angle)
        #r = s.get_rect()
        #r.center = (400, 300)
        #self.screen.blit(s, r)
        def flip():
            '''For benchmarking.'''
            pygame.display.flip()
        flip()
        if movie:
            pygame.image.save(self.screen, 'test-%04d.tga'%self.i)
        self.i += 1

    def Notify(self, event):
        if isinstance(event, Event.ObjectRefreshEvent):
            if not event.obj: return
            try:
                for geom in self.game.geometry.get_shapes()[event.obj].values():
                    #self.geom_sprites[geom] # just see if it's there
                    self.geom_sprites[geom].refresh()
                #print 'spriterefresh'
            except KeyError:
                #print 'creating new sprites'
                self.in_sprites.pop(event.obj, None)
                lst = [] # not used, but maybe for self.obj_sprites?
                geoms = self.game.geometry.get_shapes()
                for key, geom in geoms[event.obj].iteritems():
                    s = GeomSprite(geom)
                    self.geom_sprites[geom] = s 
                    lst.append(s)
                    self.spritesgroup.add(s)
                    self.layers[Constants.LAYERGEOMS].add(s)
                #self.obj_sprites[event.obj] = lst
                #print 'success'
        elif isinstance(event, Event.AddImageEvent):
            rel = event.rel
            if rel:
                rel = self.game.geometry.bodies[rel]
            txt = event.texture
            if txt:
                w, h = txt
                def texturate(i):
                    return fill_texture((w, h), 0, i)
                event.strip = Image.Strip(map(texturate, event.strip))
            s = ImageSprite(event.pos, event.strip, event.rot, event.attr,
                            rel, start=event.start, repeat=event.repeat)
            self.layers[event.layer].add(s)
            event.image = s
        elif isinstance(event, Event.GeomBGEvent):
            s = GeomSprite(event.geom, image=event.image, texture=event.texture)
            #print s
            self.layers[Constants.LAYERBG].add(s)

        elif isinstance(event, Event.ExplosionSpriteEvent):
            s = Explosion(*event.args)
            self.explodesprites.add(s)
            self.layers[Constants.LAYEREXPLOSIONS].add(s)
        elif isinstance(event, Event.ObjectDestroyEvent):
            geoms = self.game.geometry.get_shapes()
            for key, geom in geoms[event.obj].iteritems():
                s = self.geom_sprites[geom]
                del self.geom_sprites[geom]
                s.kill()
                if not event.chrome: return # the rest of fcn is chrome
                if hasattr(event.obj,'outanim') and event.obj.outanim:
                    outanim = event.obj.outanim
                    if outanim.find("%") >= 0:
                        try:
                            strip = Image.find_image(outanim % {"killer":event.killstr}, self.game.currentfile)
                        except ValueError:
                            strip = Image.find_image(outanim % {"killer":""},
                                                     self.game.currentfile)
                    else:
                        strip = Image.find_image(outanim, self.game.currentfile)
                    imgs = strip.images
                    repeat = False
                    if event.obj.remains:
                        if isinstance(event.obj.remains, Image.Strip):
                            repeat = len(strip)
                            strip.images += event.obj.remains.images
                        else:
                            repeat = len(strip)-1
                    s = GeomSprite(geom, image=Image.Strip(imgs), repeat=repeat)
                    self.layers[Constants.LAYEREXPLOSIONS].add(s)
                if hasattr(event.obj, 'corpse') and event.obj.corpse:
                    s = GeomSprite(geom, image=event.obj.corpse, repeat=True)
                    self.geom_sprites[geom] = s
                    self.layers[Constants.LAYERGEOMS].add(s)
        elif isinstance(event, Event.ObjectDestroyPartEvent):
            # event.obj is a geom
            label = event.obj.label
            if event.obj.object.extras.has_key(label):
                sprites = event.obj.object.extras[label]
                del event.obj.object.extras[label]
                for s in sprites:
                    s.kill()
            g = event.obj
            s = self.geom_sprites[g]
            del self.geom_sprites[g]
            s.kill()
        elif isinstance(event, Event.ObjectIncomingEvent):
            o = event.obj
            s = InSprite(obj, 0, self)
            self.spritesgroup.add(s)
            self.layers[Constants.LAYERGEOMS].add(s)
            self.in_sprites[o] = s
        elif isinstance(event, Event.ObjectHitEvent):
            geoms = self.game.geometry.get_shapes()
            for key, geom in geoms[event.obj].iteritems():
                s = FlashSprite(event.obj, geom, self.geom_sprites[geom])
                self.flashsprites.add(s)
                self.layers[Constants.LAYEREXPLOSIONS].add(s)
        elif isinstance(event, Event.ObjectShootingEvent):
            if not event.obj.attackimg: return
            geoms = self.game.geometry.get_shapes()
            for key, geom in geoms[event.obj].iteritems():
                s = self.geom_sprites[geom]
                s.switch(event.obj.attackimg)

        elif isinstance(event, Event.AddChainEvent):
            b1 = self.game.geometry.bodies[event.obj1]
            b2 = self.game.geometry.bodies[event.obj2]
            s = ChainSprite(b1, b2, event.pos1, event.pos2,
                            event.txt, event.rot, start=event.start)
            self.layers[Constants.LAYERBG].add(s)
        
        elif isinstance(event, Event.ScreenshotEvent):
            pygame.image.save(self.screen, 'test-%04d.tga'%self.i)

        elif isinstance(event, Event.FXEvent):
            r = self.fx_funcs[event.type](self, event.part, **event.kwargs)
            event.response = r

    textcolor = (255, 255, 255)
    def draw_fps(self):
        fps = "%d"%self.game.fps
        s = self.healthfont.render(fps, True, self.textcolor, (0, 0, 0))
        s.set_colorkey((0, 0, 0))
        self.screen.blit(s, (720, 500))

    def draw_health(self):
        healthstr = "%d"%self.game.player.health
        s = self.healthfont.render(healthstr, True, self.textcolor, (0, 0, 0))
        s.set_colorkey((0, 0, 0))
        self.screen.blit(s, (20, 500))

    def draw_goals(self, offset):
        '''This draws all the goals in self.goals.'''
        # Each goal is really a Level.Event!
        for g in self.game.goals:
            try:
                View.draw_event_funcs[g.__class__](self, g, offset)
            except KeyError:
                pass # Oh, well
        
    # A table of drawing functions for each goal.
    def draw_objectin(self, event, offset):
        area = event.area.move(offset)
        if event.rel:
            area.move_ip(self.game.geometry.get_pos(event.rel._object))
        pygame.draw.rect(self.screen, (100, 200, 100), area, 2)

    def draw_isnt(self, event, offset):
        #Uh, hmm.
        pass

    def draw_which(self, event, offset):
        #Draw them the same color?
        pass

    def draw_all(self, event, offset):
        pass

    def draw_ticks(self, event, offset):
        pass

    def draw_seconds(self, event, offset):
        pass

    def draw_kills(self, event, offset):
        pass

    def draw_noneevent(self, event, offset):
        pass

    def draw_never(self, event, offset):
        pass

    def draw_dialog(self, event, offset):
        try:
            event.sprite
        except AttributeError:
            self.render_sprite(event)
        self.screen.blit(event.window, event.windowrect)
        #self.screen.blit(event.textsprite, event.textrect) # covered in windowrect

    def draw_sequence(self, event, offset):
        pass

    def make_window(self, event):
        c = event.character
        w, h = self.screen.get_size()
        surf = pygame.Surface((int(w*.9), int(h*.1)))
        w, h = surf.get_size()
        surf.fill(c.color)
        #This style of border is taken from Starfighter.
        pygame.draw.line(surf, (200, 200, 200), (0, 0), (0, h))
        pygame.draw.line(surf, (200, 200, 200), (0, 0), (w, 0))
        pygame.draw.line(surf, (100, 100, 100), (w-1, 0), (w-1, h-1))
        pygame.draw.line(surf, (100, 100, 100), (0, h-1), (w-1, h-1))
        
        return surf

    def fx_invert(self, part, **args):
        state = not self.background.state
        self.background.strip = self.background.invertstrips[state]
        self.background.state = state
        self.background.repeat = len(self.background.strip)-1
        self.background.n = 0
        self.textcolor = (0, 0, 0)

    def fx_fade(self, part, **args):
        color = args.pop('color')
        ticks = args.pop('ticks')
        self.layers[Constants.LAYEREXPLOSIONS].add(
            FadeSprite(self.screen.get_size(), color, ticks, **args))

    def fx_rotozoom(self, part, **args):
        if isinstance(part, str):
            image = Image.find_image(part, self.game.currentfile, False)

        s = ZoomSprite(image=image, **args)
        self.layers[Constants.LAYEREXPLOSIONS].add(s)
        return s

    def render_sprite(self, event):
        event.window = self.make_window(event)
        event.windowrect = event.window.get_rect()
        def wrap_text(font, text, color, bg, w, h):
            s = pygame.Surface((w, h))
            s.fill(bg)
            words = text.split()
            def extract_line(func, limit):
                i = 0
                line = words.pop(0)
                while words and font.size(line + ' ' + words[0])[0] < limit:
                    line += ' ' + words.pop(0)
                return line

            newh = 0
            while words:
                line = extract_line(font.size, w)
                lsurf = font.render(line, True, color, bg)
                s.blit(lsurf, (0, newh))
                newh += lsurf.get_height()
            return s

        w, h = event.windowrect.size
        event.textsprite = wrap_text(self.font, event.text, (255, 255, 255),
                                     event.character.color, w-60, h-10)
        textrect = event.textsprite.get_rect()
        textrect.midtop = event.windowrect.move(29, 5).midtop
        event.window.blit(event.textsprite, textrect)
        if event.character.picture:
            image = Image.load_image(event.character.picture)
            imagerect = image.get_rect()
            imagerect.move_ip(2, 2)
            event.window.blit(image, imagerect)
        #event.window.fill((255, 255, 255), textrect)
        event.windowrect.topleft = (50, 50)

    fx_funcs = {'invert': fx_invert, 'fade': fx_fade, 'rotozoom': fx_rotozoom}


_drawfuncs = {}
for name, thing in Goals.__dict__.iteritems():
    if isinstance(thing, type):
        if issubclass(thing, Goals.Event):
            funcname = 'draw_%s'%(name.lower())
            if hasattr(View, funcname):
                _drawfuncs[thing] = getattr(View, "draw_%s"%name.lower())

View.draw_event_funcs = _drawfuncs
            
