import operator
import Image
import PhysicsController
import Game
import Rope
from Object import Object
class Bullet(Object):
    size=4
    def __init__(self, player, bounces=-1, size=None, strength=None,
                 killstr = ""):
        super(Bullet, self).__init__()
        self.angle = player.angle
        self.shooter = player
        self.bounces = bounces
        if size: self.size = size
        if isinstance(strength, int):
            self.strength = strength
        else:
            self.strength = 2
        self.killstr = killstr

    def add_geometry(self, geometry):
        pass

    def passable(self, part, o2, n, t, game):
        if self.shooter == o2: return True
        return False

    def pre_collide(self, pass1, o2, pass2, n, game, **kwargs):
        #return
        boom = False
        if self.bounces == 0 and not pass2:
            boom = True
        if not pass2:
            self.bounces -= 1
            
        game.hit(o2, self.killstr, self.strength)
        if not o2.static:
            m = game.FORCESCALE
            accel = [n[0]*self.strength*m, n[1]*self.strength*m]
            game.add_impulse(o2, accel)

        if boom:
            self.explode(game)

        return True

    def explode(self, game):
        c = game.geometry.get_pos(self)
        game.queue_delete(self)
        game.make_explosion(c, self.angle, self.shooter.bulletexplodeimage,
                            self.shooter.bulletexplodesound)
        

class Hook(Bullet):
    size = 4
    imagename = 'beamhook'
    def __init__(self, player):
        super(Hook, self).__init__(player)

    def add_geometry(self, geometry):
        self.geometry = geometry

    def pre_collide(self, o2, pass2, t, game, **kwargs):
        if pass2:
            return
        p0 = self.geometry.get_pos(self)
        v = map(lambda x: t*x, self.geometry.get_velo(self))
        pos = map(operator.add, p0, v)
        
        game.add_rope(Rope.ODERope, self.shooter, o2, pos, 100)
        #self.player.hooked = self
        game.queue_delete(self)
        
        game.make_explosion(pos, self.angle, 'beamhook')
